package hu.bme.iit.ar.opencvexample;

import android.content.pm.ActivityInfo;
import android.opengl.GLSurfaceView;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import org.opencv.android.*;
import android.util.*;


import org.opencv.android.BaseLoaderCallback;
import org.opencv.android.CameraBridgeViewBase.CvCameraViewFrame;
import org.opencv.android.LoaderCallbackInterface;
import org.opencv.android.OpenCVLoader;
import org.opencv.core.Mat;
import org.opencv.android.CameraBridgeViewBase;
import org.opencv.android.CameraBridgeViewBase.CvCameraViewListener2;

import org.opencv.core.Core;
import org.opencv.core.MatOfPoint;
import org.opencv.core.MatOfPoint2f;
import org.opencv.core.Point;
import org.opencv.core.RotatedRect;
import org.opencv.core.Scalar;
import org.opencv.imgproc.Imgproc;
import java.util.ArrayList;


import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.SurfaceView;
import android.view.WindowManager;
import android.widget.Toast;

public class MainActivity extends Activity implements CvCameraViewListener2 {
    private static final String TAG = "OCVSample::Activity";

    private CameraBridgeViewBase mOpenCvCameraView;
    private boolean              mIsJavaCamera = true;
    private MenuItem             mItemSwitchCamera = null;
    private MainSurfaceView mGLView;

    private BaseLoaderCallback mLoaderCallback = new BaseLoaderCallback(this) {
        @Override
        public void onManagerConnected(int status) {
            switch (status) {
                case LoaderCallbackInterface.SUCCESS:
                {
                    Log.i(TAG, "OpenCV loaded successfully");
                    mOpenCvCameraView.enableView();
                } break;
                default:
                {
                    super.onManagerConnected(status);
                } break;
            }
        }
    };

    public MainActivity() {
        Log.i(TAG, "Instantiated new " + this.getClass());
    }

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        Log.i(TAG, "called onCreate");
        super.onCreate(savedInstanceState);
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

        mGLView = new MainSurfaceView(this);
        setContentView(mGLView);

        mOpenCvCameraView = (CameraBridgeViewBase) new JavaCameraView(this, -1);
        mOpenCvCameraView.setCvCameraViewListener(this);
        addContentView(mOpenCvCameraView, new ActionBar.LayoutParams(
                ActionBar.LayoutParams.WRAP_CONTENT, ActionBar.LayoutParams.WRAP_CONTENT));

        //setContentView(R.layout.activity_main);
        //mOpenCvCameraView = (CameraBridgeViewBase) findViewById(R.id.opencvexample_activity_java_surface_view);
        //mOpenCvCameraView.setVisibility(SurfaceView.VISIBLE);
        //mOpenCvCameraView.setCvCameraViewListener(this);
    }

    @Override
    public void onPause()
    {
        super.onPause();
        if (mOpenCvCameraView != null)
            mOpenCvCameraView.disableView();
    }

    @Override
    public void onResume()
    {
        super.onResume();
        if (!OpenCVLoader.initDebug()) {
            Log.d(TAG, "Internal OpenCV library not found. Using OpenCV Manager for initialization");
            OpenCVLoader.initAsync(OpenCVLoader.OPENCV_VERSION_3_1_0, this, mLoaderCallback);
        } else {
            Log.d(TAG, "OpenCV library found inside package. Using it!");
            mLoaderCallback.onManagerConnected(LoaderCallbackInterface.SUCCESS);
        }
    }

    public void onDestroy() {
        super.onDestroy();
        if (mOpenCvCameraView != null)
            mOpenCvCameraView.disableView();
    }

    public void onCameraViewStarted(int width, int height) {
    }

    public void onCameraViewStopped() {
    }
/*
    public Mat onCameraFrame(CvCameraViewFrame inputFrame) {
        Mat inputMat = inputFrame.rgba();

        ArrayList<RotatedRect> minRect = new ArrayList<>();
        ArrayList<RotatedRect> minEllipse = new ArrayList<>();
        ArrayList<MatOfPoint> contours;

        Mat segmentedInput = colorSegment(inputMat);
        contours = detectContours( segmentedInput );

        detectEllipses(contours, minRect, minEllipse);

        // debug: draw what we detected
        for (int i = 0; i< contours.size(); i++)
        {
            //Scalar color = new Scalar((int)(Math.random() * 256), (int)(Math.random() * 256), (int)(Math.random() * 256)); // random color
            Scalar color = new Scalar(0, 255, 255);
            //Imgproc.drawContours(inputMat, contours, i, color, 1);
            if (i < minEllipse.size()) {
                //Imgproc.ellipse(inputMat, minEllipse.get(i), color, 2, 8);
                Point points[] = new Point[4];
                minRect.get(i).points(points);
                for (int j = 0; j < 4; j++) {
                    Imgproc.line(inputMat, points[j], points[(j + 1) % 4], color);
                }
            }
        }

        mGLView.getRenderer().setDetectedRectangles(minRect);
        mGLView.getRenderer().setVideoSize(inputMat.size());


        return inputMat;
    }
    public Mat colorSegment(Mat src) {
        Mat result = new Mat();
        Mat hsv_image = new Mat(), mask = new Mat();
        Imgproc.cvtColor(src, hsv_image, Imgproc.COLOR_RGB2HSV);
        Core.inRange(hsv_image, new Scalar(0,100,100), new Scalar(10,255,255), mask);  // red
        //Core.inRange(hsv_image, new Scalar(100,100,100), new Scalar(110,255,255), mask);  // blue
        result = Mat.zeros(src.size(),src.type());
        src.copyTo(result, mask);
        return result;
    }

    public ArrayList<MatOfPoint> detectContours(Mat src) {
        Mat src_gray = new Mat(), canny_output = new Mat();
        ArrayList<MatOfPoint> contours = new ArrayList<>();

        Imgproc.cvtColor(src, src_gray, Imgproc.COLOR_BGR2GRAY);

        double threshold = 100.0;
        Imgproc.Canny(src_gray, canny_output, threshold, threshold * 2.0);

        Mat hierarchy = new Mat();
        Imgproc.findContours(canny_output, contours, hierarchy, Imgproc.RETR_TREE, Imgproc.CHAIN_APPROX_SIMPLE, new Point(0,0));

        return contours;
    }

    public void detectEllipses(ArrayList<MatOfPoint> contours, ArrayList<RotatedRect> minRect, ArrayList<RotatedRect> minEllipse) {

        for (int i = 0; i < contours.size(); i++)
        {
            MatOfPoint2f contoursF = new MatOfPoint2f( contours.get(i).toArray() );
            minRect.add(Imgproc.minAreaRect(contoursF));
            if (contours.get(i).rows() > 5) {
                minEllipse.add(Imgproc.fitEllipse(contoursF));
            }
        }
    }
*/

    public Mat onCameraFrame(CvCameraViewFrame inputFrame) {
        Mat inputMat = inputFrame.rgba();

        ArrayList<RotatedRect> minRect = new ArrayList<>();
        ArrayList<RotatedRect> minEllipse = new ArrayList<>();
        ArrayList<MatOfPoint> contours;

        Mat segmentedInput = colorSegment(inputMat);
        contours = detectContours( segmentedInput );

        detectEllipses(contours, minRect, minEllipse);

        // debug: draw what we detected
        for (int i = 0; i< contours.size(); i++)
        {
            //Scalar color = new Scalar((int)(Math.random() * 256), (int)(Math.random() * 256), (int)(Math.random() * 256)); // random color
            Scalar color = new Scalar(0, 255, 255);
            //Imgproc.drawContours(inputMat, contours, i, color, 1);
            if (i < minEllipse.size()) {
                //Imgproc.ellipse(inputMat, minEllipse.get(i), color, 2, 8);
                Point points[] = new Point[4];
                minRect.get(i).points(points);
                for (int j = 0; j < 4; j++) {
                    Imgproc.line(inputMat, points[j], points[(j + 1) % 4], color);
                }
            }
        }

        mGLView.getRenderer().setDetectedRectangles(minRect);
        mGLView.getRenderer().setVideoSize(inputMat.size());


        return inputMat;
    }

    Mat result;
    Mat hsv_image, mask;
    Mat src_gray, canny_output;
    Mat hierarchy;
    public Mat colorSegment(Mat src) {
        if (result == null) result = new Mat();
        if (hsv_image == null) hsv_image = new Mat();
        if (mask == null) mask = new Mat();

        Imgproc.cvtColor(src, hsv_image, Imgproc.COLOR_RGB2HSV);
        Core.inRange(hsv_image, new Scalar(0,100,100), new Scalar(10,255,255), mask);  // red
        //Core.inRange(hsv_image, new Scalar(100,100,100), new Scalar(110,255,255), mask);  // blue
        result = Mat.zeros(src.size(),src.type());
        src.copyTo(result, mask);
        return result;
    }

    public ArrayList<MatOfPoint> detectContours(Mat src) {
        if (src_gray == null) src_gray = new Mat();
        if (canny_output == null) canny_output = new Mat();
        ArrayList<MatOfPoint> contours = new ArrayList<>();

        Imgproc.cvtColor(src, src_gray, Imgproc.COLOR_BGR2GRAY);

        double threshold = 100.0;
        Imgproc.Canny(src_gray, canny_output, threshold, threshold * 2.0);

        if (hierarchy == null) hierarchy = new Mat();
        Imgproc.findContours(canny_output, contours, hierarchy, Imgproc.RETR_TREE, Imgproc.CHAIN_APPROX_SIMPLE, new Point(0,0));

        return contours;
    }

    public void detectEllipses(ArrayList<MatOfPoint> contours, ArrayList<RotatedRect> minRect, ArrayList<RotatedRect> minEllipse) {

        for (int i = 0; i < contours.size(); i++)
        {
            MatOfPoint2f contoursF = new MatOfPoint2f( contours.get(i).toArray() );
            minRect.add(Imgproc.minAreaRect(contoursF));
            if (contours.get(i).rows() > 5) {
                minEllipse.add(Imgproc.fitEllipse(contoursF));
            }
        }
    }
}