package hu.bme.iit.ar.opencvexample;

/**
 * Created by umitomi on 10/22/2016.
 */
import android.content.Context;
import android.opengl.GLSurfaceView;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.content.res.AssetManager;

import org.opencv.core.RotatedRect;
import org.opencv.core.Size;

import java.util.ArrayList;

public class MainRenderer implements GLSurfaceView.Renderer {
    private Context context;
    private int windowWidth = 0;
    private int windowHeight = 0;
    private Size videoSize=null;

    Material m;
    Texture t;
    Mesh mesh;
    ObjMesh objMesh;
    Material objMeshMat;
    public float rotAngle = 0;
    public AssetManager assetManager = null;
    ArrayList<RotatedRect> detectedRectangles = new ArrayList<>();

    public void setDetectedRectangles(ArrayList<RotatedRect> rects) {
        synchronized(detectedRectangles) {
            detectedRectangles.clear();
            detectedRectangles.addAll(rects);
        }

    }

    public void setVideoSize(Size s){videoSize = s;}

    public MainRenderer(Context context) {
        this.context = context;
        assetManager = context.getAssets();

    }

    public void onSurfaceCreated(GL10 gl, EGLConfig config) {
        Globals.assetManager = assetManager;

        gl.glClearColor(0, 0, 0, 0);
        gl.glClearDepthf(1.0f);
        gl.glEnable(gl.GL_DEPTH_TEST);

        t = new Texture("TestTexture");
        t.setFileName("brick.jpg");
        t.load();
        m = new Material("TestMaterial");
        m.setTexture(t);
        m.load();
        mesh = new Quad("TestQuad");
        mesh.load();

        objMesh = new ObjMesh("car");
        objMesh.setFileName("stanfordBunny.obj");
        objMesh.load();
        objMeshMat = new Material("CarMaterial");
        objMeshMat.setLighting(true);
        objMeshMat.setColor(1.0f,0.5f,0.2f,1.0f);
        objMeshMat.load();

    }

    public void onDrawFrame(GL10 gl) {

        if(videoSize == null)
            videoSize = new Size(windowWidth, windowHeight);
        float ratio = (float) (videoSize.width / videoSize.height);
        gl.glViewport(windowWidth / 2 - (int) videoSize.width / 2, windowHeight / 2 - (int) videoSize.height / 2, (int) videoSize.width, (int) videoSize.height);
        gl.glMatrixMode(gl.GL_PROJECTION);
        gl.glLoadIdentity();
        float frontPlane = 1.0f;
        float frustumRight = ratio;
        float frustumTop = 1.0f;
        gl.glFrustumf(-frustumRight, frustumRight, -frustumTop, frustumTop, frontPlane, 1000);

        gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT );

        gl.glMatrixMode(gl.GL_MODELVIEW);
        gl.glLoadIdentity();

        gl.glEnable(gl.GL_LIGHT0);
        float lightColor[] = {1,1,1,1};
        gl.glLightfv(gl.GL_LIGHT0,gl.GL_DIFFUSE,lightColor,0);
        gl.glLightfv(gl.GL_LIGHT0,gl.GL_SPECULAR,lightColor,0);
        float ambientLightColor[] = {1,1,1,1};
        gl.glLightfv(gl.GL_LIGHT0,gl.GL_AMBIENT,ambientLightColor,0);
        float lightPos[] = {0,0,0,1};
        gl.glLightfv(gl.GL_LIGHT0,gl.GL_POSITION, lightPos,0);

        objMeshMat.set();

       synchronized(detectedRectangles) {
            for (int i = 0; i < detectedRectangles.size(); ++i) {
                float x = (float) detectedRectangles.get(i).center.x;
                float y = (float) detectedRectangles.get(i).center.y;
                //convert pixel coordinates to screen(-1 - 1)
                x /= (float) videoSize.width;
                x *= 2.0;
                x -= 1.0;
                y /= (float) videoSize.height;
                y *= 2.0;
                y -= 1.0;
                x *= frustumRight;
                y *= -frustumTop;//also flip y
                float z = -frontPlane;
                //set distance according to rectangle size
                float markerWidthSS = (float) detectedRectangles.get(i).size.width / (float) videoSize.width;
                float zratio = 0.5f / markerWidthSS;
                x *= zratio;
                y *= zratio;
                z *= zratio;

                gl.glPushMatrix();
                gl.glTranslatef(x, y, z);
                //gl.glRotatef(rotAngle*2, 0, 1, 0);
                //gl.glScalef(0.05f,0.05f,0.05f);
                //gl.glScalef(0.1f,0.1f,0.1f);
                objMesh.render();
                gl.glPopMatrix();
            }
        }

        rotAngle += 2.5f;
    }

    public void onSurfaceChanged(GL10 gl, int width, int height) {
        windowWidth = width;
        windowHeight = height;
        gl.glViewport(0, 0, width, height);
        float ratio = (float) width / height;
        gl.glMatrixMode(gl.GL_PROJECTION);
        gl.glLoadIdentity();
        gl.glFrustumf(-0.1f*ratio, 0.1f*ratio, 0.1f, 0.1f, 0.1f, 1000);

    }
}

