package hu.bme.iit.ar.opencvexample;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Vector;

public class ObjMesh extends Mesh
{
  protected String filename = null;
  private String tokens;

  public ObjMesh(String name){
    super(name);
  }

  public void setFileName(String filename) {
    this.filename = filename;
  }

  public void load()
  {
    InputStream is;
    try {
      is = Globals.assetManager.open(filename);
    }
    catch(IOException e){
      e.printStackTrace();
      return;
    }

    BufferedReader br = new BufferedReader(new java.io.InputStreamReader(is));
    String line;
    Vector<Float> vertices = new  Vector<Float>();
    Vector<Float> texCoords = new  Vector<Float>();
    Vector<Float> normals = new  Vector<Float>();
    class Face
    {
      int v[] = new int[3];
      int t[] = new int[3];
      int n[] = new int[3];
    };
    Vector<Face> faces = new  Vector<Face>();

    try
    {
      line = br.readLine();
      String[] tokens;
      String[] tokens2;
      while(line != null)
      {
        tokens =  line.split(" ");
        if(tokens.length == 0 || tokens[0] == "")
        {
          line = br.readLine();
          continue;
        }
        if(tokens[0].compareTo("v") == 0)
        {
          float f = Float.parseFloat(tokens[1]);
          vertices.add(f);
          f = Float.parseFloat(tokens[2]);
          vertices.add(f);
          f = Float.parseFloat(tokens[3]);
          vertices.add(f);
        }
        else if(tokens[0].compareTo("vt") == 0)
        {
          float f = Float.parseFloat(tokens[1]);
          texCoords.add(f);
          f = Float.parseFloat(tokens[2]);
          texCoords.add(f);
        }
        else if(tokens[0].compareTo("vn") == 0)
        {
          float f = Float.parseFloat(tokens[1]);
          normals.add(f);
          f = Float.parseFloat(tokens[2]);
          normals.add(f);
          f = Float.parseFloat(tokens[3]);
          normals.add(f);
        }
        else if(tokens[0].charAt(0) == 'f')
        {
          Face face = new Face();
          tokens =  line.split(" ");
          tokens2 = tokens[1].split("/");
          face.v[0] = Integer.parseInt(tokens2[0])-1;
          face.t[0] = Integer.parseInt(tokens2[1])-1;
          face.n[0] = Integer.parseInt(tokens2[2])-1;
          tokens2 = tokens[2].split("/");
          face.v[1] = Integer.parseInt(tokens2[0])-1;
          face.t[1] = Integer.parseInt(tokens2[1])-1;
          face.n[1] = Integer.parseInt(tokens2[2])-1;
          tokens2 = tokens[3].split("/");
          face.v[2] = Integer.parseInt(tokens2[0])-1;
          face.t[2] = Integer.parseInt(tokens2[1])-1;
          face.n[2] = Integer.parseInt(tokens2[2])-1;
          faces.add(face);
        }
        line = br.readLine();
      }
    }
    catch(IOException e){
      e.printStackTrace();
      return;
    }

    vertexcount = faces.size() * 3;
    float[] VCoords = new float[vertexcount * 3];
    float[] TCoords = new float[vertexcount * 2];
    float[] NCoords = new float[vertexcount * 3];
    for(int f = 0; f < faces.size(); ++f)
    {
      for(int v = 0; v < 3; ++v)
        for (int c = 0; c < 3; ++c) {
          VCoords[f * 3 * 3 + v * 3 + c] = vertices.elementAt(3 * faces.elementAt(f).v[v] + c);
          NCoords[f * 3 * 3 + v * 3 + c] = normals.elementAt(3 * faces.elementAt(f).n[v] + c);
          if (c < 2)
            TCoords[f * 3 * 2 + v * 2 + c] = texCoords.elementAt(2 * faces.elementAt(f).t[v] + c);
        }
    }

    ByteBuffer vbb = ByteBuffer.allocateDirect(VCoords.length * 4);
    vbb.order(ByteOrder.nativeOrder());
    this.VB = vbb.asFloatBuffer();
    this.VB.put(VCoords);
    this.VB.position(0);

    ByteBuffer nbb = ByteBuffer.allocateDirect(NCoords.length * 4);
    nbb.order(ByteOrder.nativeOrder());
    this.NB = nbb.asFloatBuffer();
    this.NB.put(NCoords);
    this.NB.position(0);

    ByteBuffer tbb = ByteBuffer.allocateDirect(TCoords.length * 4);
    tbb.order(ByteOrder.nativeOrder());
    this.TB = tbb.asFloatBuffer();
    this.TB.put(TCoords);
    this.TB.position(0);
  }  
}


